[−][src]Crate threshold
Re-exports
pub use crate::above_exset::AboveExSet; |
pub use crate::below_exset::BelowExSet; |
pub use crate::clock::AEClock; |
pub use crate::clock::BEClock; |
pub use crate::clock::Clock; |
pub use crate::clock::Dot; |
pub use crate::clock::VClock; |
pub use crate::maxset::MaxSet; |
pub use crate::multiset::MultiSet; |
pub use crate::tclock::TClock; |
Modules
| above_exset | This module contains an implementation of an above-extra set. |
| below_exset | This module contains an implementation of a below-exception set. |
| clock | This module contains an implementation of a vector clock. |
| maxset | This module contains an implementation of a max set. |
| multiset | This module contains an implementation of a multi-set supporting a threshold operation. |
| tclock | This module contains an implementation of a threshold clock. |
Traits
| Actor | Actor trait to be used in |
| Count | Count trait to be used in |
| EventSet | EventSet trait to be implemented by |