Module threshold::clock [−][src]
This module contains an implementation of a vector clock.
The implementation is inspired in rust-crdt’s implementation.
Examples
use threshold::*; let actor_a = "A"; let mut clock_a = VClock::new(); let mut clock_b = VClock::new(); clock_a.next(&actor_a); let event = clock_a.next(&actor_a); clock_b.join(&clock_a); assert!(clock_b.contains(&actor_a, event));
Structs
| Clock | |
| IntoIter | |
| Iter | |
| IterMut |
Functions
| vclock_from_seqs | Creates a new vector clock from a list of sequences.
|
Type Definitions
| AEClock | |
| ARClock | |
| BEClock | |
| VClock |